Upgrading Cluster Components (CAPVCD, CPI/CCM, CSI, RDE Projector)
------------------------------------------------------------------
CSE 4.2.1 and below clusters running older version of CAPVCD (1.3.0 or below), CPI/CCM (1.6.0 or below), CSI (1.6.0 or below), RDE-Projector (0.7.0) can be upgraded to run CAPVCD (1.3.0), CPI/CCM (1.6.0), CSI (1.6.0) and RDE-Projector (0.7.1) via the script upgrade_cluster_components.sh.

Pre-requisite: The script can be run on a machine that has
* Ubuntu GNU/Linux 20.04
* Cluster(s) kubeconfig available (file path will be required by cluster upgrade script)
    * If multiple clusters need to be upgraded, ensure all kubeconfig are easily accessible
* `kubectl` installed
    * Linux installation documentation: https://kubernetes.io/docs/tasks/tools/install-kubectl-linux/
* `docker` installed and running
    * Linux installation documentation: https://docs.docker.com/engine/install/ubuntu/
    * If `docker` is installed, check to see if `docker` is running by executing command `systemctl status docker`, it should indicate that it is active and running
    * If `docker` is not running, execute command `systemctl start docker`
    * If the private registry is using self-signed certificates, please add the certificates to the system's trust-store and restart `docker` with `systemctl restart docker`

Downloading and extracting the upgrade script container
-------------------------------------------------------
* Ensure `$HOME` directory is set before continuing, if not please set it using: `export HOME=<directory of choice>`
    * To confirm `$HOME` is set, execute command: `echo $HOME` to print the `$HOME` directory
* Create a folder structure for the cluster upgrade script content with:
    * `mkdir -p $HOME/cluster-upgrade-script`
* Pull and extract the cluster-upgrade-script-airgapped image from projects.registry.vmware.com with:
    * `cd $HOME/cluster-upgrade-script`
    * `docker pull projects.registry.vmware.com/vmware-cloud-director/cluster-upgrade-script-airgapped:v0.1.3`
    * `docker container create --name "temp_container" projects.registry.vmware.com/vmware-cloud-director/cluster-upgrade-script-airgapped:v0.1.3`
    * `docker export "temp_container" | tar -C $HOME/cluster-upgrade-script -xvf - --strip-components 2`
* Remove temp container that was created:
    * `docker container remove "temp_container"`

Script Usage
------------
1. Navigate to the folder path where the script was extracted: `cd $HOME/cluster-upgrade-script`
2. Set executable permission with: `chmod +x upgrade_cluster_components.sh`
3. Execute the cluster upgrade script with: `./upgrade_cluster_components.sh kubeconfig-absolute-file-path [image_registry]`
    * `image_registry` is an optional parameter, which defines the registry the script should pull images from and expects images such as CSI/CPI CRS, CAPVCD manifests, clusterctl, etc. to be hosted there.
    * When not using private/local/airgap registry, please use `projects.registry.vmware.com` as the `image_registry`
        * Alternatively if the parameter is empty, it will also default to registry: `projects.registry.vmware.com`
    * For private/local registry, please use your registry link for the parameter, such as `my-private.registry.com`
        * Please make sure that the VM on which you will be running the scripts trust the registry, or it may run into errors such as `x509 certificate signed by unknown authority`
4. If multiple clusters need to be upgraded, repeat step 3 with additional cluster kubeconfig

* Note: During script execution, all image artifacts are downloaded to `$HOME/cluster-upgrade-packages/`
    * If there are any errors during the upgrade from the script, it is safe to remove this folder and re-executing the script recreate this folder
    * Additional folder is created for clusterctl at `$HOME/.cluster-api`, which is also safe to delete as re-executing the script will recreate it

Example Usage)

For non-private registry:
* `./upgrade_cluster_components.sh $HOME/kubeconfig1.txt/ projects.registry.vmware.com`

For private/local/airgap registry:
* `./upgrade_cluster_components.sh $HOME/kubeconfig2.txt/ my-private.registry.com`